#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from zope.container.interfaces import IContained

# import packages
from onf_website.reference.forest.schema import ForestsListField
from onf_website.reference.orga.schema import StructuresListField
from pyams_utils.schema import DottedDecimalField
from zope.interface import invariant, Interface, Attribute, Invalid
from zope.schema import Bool, Set, Choice, Text, TextLine
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from onf_website import _


LOCATION_INFO_KEY = 'onf_website.location'


WGS84 = 4326
RGF93 = 2154
LAMBERT_IIE = 27572
UTM_20N = 4559
UTM_22N = 2972
UTM_38S = 4471
UTM_40S = 2975

COORDINATES_PROJECTIONS = {WGS84: _("WGS84 (GPS)"),
                           RGF93: _("Lambert 93 (Metropolitan France)"),
                           LAMBERT_IIE: _("Extended Lambert II (Metropolitan France)"),
                           UTM_20N: _("UTM Zone 20N (Martinique, Guadeloupe)"),
                           UTM_22N: _("UTM Zone 22N (Guyane)"),
                           UTM_38S: _("UTM Zone 38S (Mayotte)"),
                           UTM_40S: _("UTM Zone 40S (La Réunion)")}

COORDINATES_PROJECTION_VOCABULARY = SimpleVocabulary(sorted([SimpleTerm(v, title=t)
                                                             for v, t in COORDINATES_PROJECTIONS.items()],
                                                            key=lambda x: x.title))


class ILocationInfo(IContained):
    """Location information interface"""

    can_inherit = Attribute("Can inherit location?")

    inherit = Bool(title=_("Inherit location?"),
                   required=False,
                   default=True)

    no_inherit = Bool(title=_("Don't inherit location?"),
                      required=False,
                      default=False)

    countries = Set(title=_("Countries"),
                    value_type=Choice(vocabulary='ONF countries'),
                    required=False)

    countries_index = Attribute("Countries list used for indexing")

    departments = Set(title=_("Departments"),
                      value_type=Choice(vocabulary='ONF departments'),
                      required=False)

    departments_index = Attribute("Departments list used for indexing")

    structures = StructuresListField(title=_("Structures"),
                                     required=False)

    address = Text(title=_("Address"),
                   required=False)

    postal_code = TextLine(title=_("Postal code"),
                           required=False)

    city = TextLine(title=_("City"),
                    required=False)

    situation = TextLine(title=_("Relative situation"),
                         description=_("For example: 25km north-west from Paris"),
                         required=False)

    natural_region = TextLine(title=_("Natural region"),
                              required=False)

    forests = ForestsListField(title=_("Managed forests"),
                               required=False)

    forests_label = TextLine(title=_("Substitution label"),
                             required=False)

    projection = Choice(title=_("Projection system"),
                        vocabulary=COORDINATES_PROJECTION_VOCABULARY,
                        required=False)

    longitude = DottedDecimalField(title=_("Longitude"),
                                   required=False)

    latitude = DottedDecimalField(title=_("Latitude"),
                                  required=False)

    @invariant
    def check_coordinates(self):
        if (self.longitude and not self.latitude) or (self.latitude and not self.longitude):
            raise Invalid(_("You must set longitude and latitude!"))
        if (self.longitude or self.latitude) and not self.projection:
            raise Invalid(_("You can't set coordinates without setting projection!"))

    altitude = DottedDecimalField(title=_("Altitude"),
                                  description=_("Location altitude, given in meters"),
                                  required=False)

    def get_coordinates(self, projection=WGS84):
        """Get coordinates translated to given projection"""


class ILocationTarget(Interface):
    """Location marker interface"""
